@@ -80,6 +80,19 @@
 		<para>This function uses the same DTMF mode naming as the dtmf_mode configuration option</para>
 	</description>
 </function>
+<function name="PJSIP_MOH_PASSTHROUGH" language="en_US">
+	<synopsis>
+		Get or change the on-hold behavior for a SIP call.
+	</synopsis>
+	<syntax>
+	</syntax>
+	<description>
+		<para>When read, returns the current moh passthrough mode</para>
+		<para>When written, sets the current moh passthrough mode</para>
+		<para>If <replaceable>yes</replaceable>, on-hold re-INVITEs are sent. If <replaceable>no</replaceable>, music on hold is generated.</para>
+		<para>This function can be used to override the moh_passthrough configuration option</para>
+	</description>
+</function>
 <function name="PJSIP_SEND_SESSION_REFRESH" language="en_US">
 	<synopsis>
 		W/O: Initiate a session refresh via an UPDATE or re-INVITE on an established media session
@@ -1124,7 +1137,7 @@ static int parse_uri_cb(void *data)
 
 	pj_strdup2_with_null(pool, &tmp, args->uri);
 	uri = (pjsip_name_addr *)pjsip_parse_uri(pool, tmp.ptr, tmp.slen, PJSIP_PARSE_URI_AS_NAMEADDR);
-	if (!uri) {
+	if (!uri || (!PJSIP_URI_SCHEME_IS_SIP(uri) && !PJSIP_URI_SCHEME_IS_SIPS(uri))) {
 		ast_log(LOG_WARNING, "Failed to parse URI '%s'\n", args->uri);
 		pjsip_endpt_release_pool(ast_sip_get_pjsip_endpoint(), pool);
 		args->ret = -1;
@@ -1222,7 +1235,7 @@ static int media_offer_read_av(struct ast_sip_session *session, char *buf,
 	struct ast_stream_topology *topology;
 	int idx;
 	struct ast_stream *stream = NULL;
-	struct ast_format_cap *caps;
+	const struct ast_format_cap *caps;
 	size_t accum = 0;
 
 	if (session->inv_session->dlg->state == PJSIP_DIALOG_STATE_ESTABLISHED) {
@@ -1339,9 +1352,19 @@ static int media_offer_write_av(void *obj)
 	if (!stream) {
 		return 0;
 	}
-	caps = ast_stream_get_formats(stream);
+
+	caps = ast_format_cap_alloc(AST_FORMAT_CAP_FLAG_DEFAULT);
+	if (!caps) {
+		return -1;
+	}
+
+	ast_format_cap_append_from_cap(caps, ast_stream_get_formats(stream),
+		AST_MEDIA_TYPE_UNKNOWN);
 	ast_format_cap_remove_by_type(caps, data->media_type);
 	ast_format_cap_update_by_allow_disallow(caps, data->value, 1);
+	ast_stream_set_formats(stream, caps);
+	ast_stream_set_metadata(stream, "pjsip_session_refresh", "force");
+	ao2_ref(caps, -1);
 
 	return 0;
 }
@@ -1431,6 +1454,34 @@ int pjsip_acf_dtmf_mode_read(struct ast_channel *chan, const char *cmd, char *da
 	return 0;
 }
 
+int pjsip_acf_moh_passthrough_read(struct ast_channel *chan, const char *cmd, char *data, char *buf, size_t len)
+{
+	struct ast_sip_channel_pvt *channel;
+
+	if (!chan) {
+		ast_log(LOG_WARNING, "No channel was provided to %s function.\n", cmd);
+		return -1;
+	}
+
+	if (len < 3) {
+		ast_log(LOG_WARNING, "%s: buffer too small\n", cmd);
+		return -1;
+	}
+
+	ast_channel_lock(chan);
+	if (strcmp(ast_channel_tech(chan)->type, "PJSIP")) {
+		ast_log(LOG_WARNING, "Cannot call %s on a non-PJSIP channel\n", cmd);
+		ast_channel_unlock(chan);
+		return -1;
+	}
+
+	channel = ast_channel_tech_pvt(chan);
+	strncpy(buf, AST_YESNO(channel->session->moh_passthrough), len);
+
+	ast_channel_unlock(chan);
+	return 0;
+}
+
 struct refresh_data {
 	struct ast_sip_session *session;
 	enum ast_sip_session_refresh_method method;
@@ -1574,6 +1625,30 @@ int pjsip_acf_dtmf_mode_write(struct ast_channel *chan, const char *cmd, char *d
 	return ast_sip_push_task_wait_serializer(channel->session->serializer, dtmf_mode_refresh_cb, &rdata);
 }
 
+int pjsip_acf_moh_passthrough_write(struct ast_channel *chan, const char *cmd, char *data, const char *value)
+{
+	struct ast_sip_channel_pvt *channel;
+
+	if (!chan) {
+		ast_log(LOG_WARNING, "No channel was provided to %s function.\n", cmd);
+		return -1;
+	}
+
+	ast_channel_lock(chan);
+	if (strcmp(ast_channel_tech(chan)->type, "PJSIP")) {
+		ast_log(LOG_WARNING, "Cannot call %s on a non-PJSIP channel\n", cmd);
+		ast_channel_unlock(chan);
+		return -1;
+	}
+
+	channel = ast_channel_tech_pvt(chan);
+	channel->session->moh_passthrough = ast_true(value);
+
+	ast_channel_unlock(chan);
+
+	return 0;
+}
+
 static int refresh_write_cb(void *obj)
 {
 	struct refresh_data *data = obj;
