@@ -82,11 +82,18 @@
 
 /*! Maximum application/json or application/x-www-form-urlencoded body content length. */
 #if !defined(LOW_MEMORY)
-#define MAX_CONTENT_LENGTH 4096
+#define MAX_CONTENT_LENGTH 40960
 #else
 #define MAX_CONTENT_LENGTH 1024
 #endif	/* !defined(LOW_MEMORY) */
 
+/*! Initial response body length. */
+#if !defined(LOW_MEMORY)
+#define INITIAL_RESPONSE_BODY_BUFFER 1024
+#else
+#define INITIAL_RESPONSE_BODY_BUFFER 512
+#endif	/* !defined(LOW_MEMORY) */
+
 /*! Maximum line length for HTTP requests. */
 #if !defined(LOW_MEMORY)
 #define MAX_HTTP_LINE_LENGTH 4096
@@ -132,7 +139,8 @@ static AST_RWLIST_HEAD_STATIC(uris, ast_http_uri);	/*!< list of supported handle
 
 /* all valid URIs must be prepended by the string in prefix. */
 static char prefix[MAX_PREFIX];
-static int enablestatic;
+static int static_uri_enabled;
+static int status_uri_enabled;
 
 /*! \brief Limit the kinds of files we're willing to serve up */
 static struct {
@@ -253,9 +261,13 @@ static int static_callback(struct ast_tcptls_session_instance *ser,
 		return 0;
 	}
 
-	/* Yuck.  I'm not really sold on this, but if you don't deliver static content it makes your configuration
-	   substantially more challenging, but this seems like a rather irritating feature creep on Asterisk. */
-	if (!enablestatic || ast_strlen_zero(uri)) {
+	/* Yuck.  I'm not really sold on this, but if you don't deliver static content it
+	 * makes your configuration substantially more challenging, but this seems like a
+	 * rather irritating feature creep on Asterisk.
+	 *
+	 * XXX: It is not clear to me what this comment means or if it is any longer
+	 *      relevant. */
+	if (ast_strlen_zero(uri)) {
 		goto out403;
 	}
 
@@ -406,7 +418,7 @@ static int httpstatus_callback(struct ast_tcptls_session_instance *ser,
 	return 0;
 }
 
-static struct ast_http_uri statusuri = {
+static struct ast_http_uri status_uri = {
 	.callback = httpstatus_callback,
 	.description = "Asterisk HTTP General Status",
 	.uri = "httpstatus",
@@ -415,7 +427,7 @@ static struct ast_http_uri statusuri = {
 	.key = __FILE__,
 };
 
-static struct ast_http_uri staticuri = {
+static struct ast_http_uri static_uri = {
 	.callback = static_callback,
 	.description = "Asterisk HTTP Static Delivery",
 	.uri = "static",
@@ -557,7 +569,7 @@ void ast_http_create_response(struct ast_tcptls_session_instance *ser, int statu
 {
 	char server_name[MAX_SERVER_NAME_LENGTH];
 	struct ast_str *server_address = ast_str_create(MAX_SERVER_NAME_LENGTH);
-	struct ast_str *out = ast_str_create(MAX_CONTENT_LENGTH);
+	struct ast_str *out = ast_str_create(INITIAL_RESPONSE_BODY_BUFFER);
 
 	if (!http_header_data || !server_address || !out) {
 		ast_free(http_header_data);
@@ -916,14 +928,24 @@ void ast_http_body_read_status(struct ast_tcptls_session_instance *ser, int read
 static int http_body_read_contents(struct ast_tcptls_session_instance *ser, char *buf, int length, const char *what_getting)
 {
 	int res;
+	int total = 0;
 
 	/* Stream is in exclusive mode so we get it all if possible. */
-	res = ast_iostream_read(ser->stream, buf, length);
-	if (res < length) {
-		ast_log(LOG_WARNING, "Short HTTP request %s (Wanted %d)\n",
-			what_getting, length);
+	while (total != length) {
+		res = ast_iostream_read(ser->stream, buf + total, length - total);
+		if (res <= 0) {
+			break;
+		}
+
+		total += res;
+	}
+
+	if (total != length) {
+		ast_log(LOG_WARNING, "Wrong HTTP content read. Request %s (Wanted %d, Read %d)\n",
+			what_getting, length, res);
 		return -1;
 	}
+
 	return 0;
 }
 
@@ -2003,6 +2025,7 @@ static void add_redirect(const char *value)
 	struct http_uri_redirect *redirect, *cur;
 	unsigned int target_len;
 	unsigned int total_len;
+	size_t dest_len;
 
 	dest = ast_strdupa(value);
 	dest = ast_skip_blanks(dest);
@@ -2016,14 +2039,15 @@ static void add_redirect(const char *value)
 	}
 
 	target_len = strlen(target) + 1;
-	total_len = sizeof(*redirect) + target_len + strlen(dest) + 1;
+	dest_len = strlen(dest) + 1;
+	total_len = sizeof(*redirect) + target_len + dest_len;
 
 	if (!(redirect = ast_calloc(1, total_len))) {
 		return;
 	}
 	redirect->dest = redirect->target + target_len;
 	strcpy(redirect->target, target);
-	strcpy(redirect->dest, dest);
+	ast_copy_string(redirect->dest, dest, dest_len);
 
 	AST_RWLIST_WRLOCK(&uri_redirects);
 
@@ -2054,8 +2078,9 @@ static int __ast_http_load(int reload)
 {
 	struct ast_config *cfg;
 	struct ast_variable *v;
-	int enabled=0;
-	int newenablestatic=0;
+	int enabled = 0;
+	int new_static_uri_enabled = 0;
+	int new_status_uri_enabled = 1; /* Default to enabled for BC */
 	char newprefix[MAX_PREFIX] = "";
 	char server_name[MAX_SERVER_NAME_LENGTH];
 	struct http_uri_redirect *redirect;
@@ -2133,8 +2158,10 @@ static int __ast_http_load(int reload)
 			}
 		} else if (!strcasecmp(v->name, "enabled")) {
 			enabled = ast_true(v->value);
-		} else if (!strcasecmp(v->name, "enablestatic")) {
-			newenablestatic = ast_true(v->value);
+		} else if (!strcasecmp(v->name, "enablestatic") || !strcasecmp(v->name, "enable_static")) {
+			new_static_uri_enabled = ast_true(v->value);
+		} else if (!strcasecmp(v->name, "enable_status")) {
+			new_status_uri_enabled = ast_true(v->value);
 		} else if (!strcasecmp(v->name, "bindport")) {
 			if (ast_parse_arg(v->value, PARSE_UINT32 | PARSE_IN_RANGE | PARSE_DEFAULT,
 				&bindport, DEFAULT_PORT, 0, 65535)) {
@@ -2185,7 +2212,6 @@ static int __ast_http_load(int reload)
 	}
 
 	ast_copy_string(http_server_name, server_name, sizeof(http_server_name));
-	enablestatic = newenablestatic;
 
 	if (num_addrs && enabled) {
 		int i;
@@ -2231,6 +2257,22 @@ static int __ast_http_load(int reload)
 		}
 	}
 
+	if (static_uri_enabled && !new_static_uri_enabled) {
+		ast_http_uri_unlink(&static_uri);
+	} else if (!static_uri_enabled && new_static_uri_enabled) {
+		ast_http_uri_link(&static_uri);
+	}
+
+	static_uri_enabled = new_static_uri_enabled;
+
+	if (status_uri_enabled && !new_status_uri_enabled) {
+		ast_http_uri_unlink(&status_uri);
+	} else if (!status_uri_enabled && new_status_uri_enabled) {
+		ast_http_uri_link(&status_uri);
+	}
+
+	status_uri_enabled = new_status_uri_enabled;
+
 	return 0;
 }
 
@@ -2312,8 +2354,13 @@ static int unload_module(void)
 	ast_free(http_tls_cfg.pvtfile);
 	ast_free(http_tls_cfg.cipher);
 
-	ast_http_uri_unlink(&statusuri);
-	ast_http_uri_unlink(&staticuri);
+	if (status_uri_enabled) {
+		ast_http_uri_unlink(&status_uri);
+	}
+
+	if (static_uri_enabled) {
+		ast_http_uri_unlink(&static_uri);
+	}
 
 	AST_RWLIST_WRLOCK(&uri_redirects);
 	while ((redirect = AST_RWLIST_REMOVE_HEAD(&uri_redirects, entry))) {
@@ -2326,8 +2373,6 @@ static int unload_module(void)
 
 static int load_module(void)
 {
-	ast_http_uri_link(&statusuri);
-	ast_http_uri_link(&staticuri);
 	ast_cli_register_multiple(cli_http, ARRAY_LEN(cli_http));
 
 	return __ast_http_load(0) ? AST_MODULE_LOAD_FAILURE : AST_MODULE_LOAD_SUCCESS;
